
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of prefswitch.

Prefswitch is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Prefswitch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with prefswitch; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

const BOOLEAN = "boolean";
const INTEGER = "integer";
const STRING = "string";
const RESET = "reset";
const LIST = "list";
const LISTITEMS = "listitems";

const UNKNOWN = 0;
const CSS = 1;
const JS = 2;

const UPDATE_ALL = "updateAll";
const UPDATE_ONE = "updateOne";
const UPDATE_PROFILES = "updateProfiles";
const LOAD_PROFILE = "loadProfile";

const ROOT = "extensions.prefswitch.";
const VALUE = "value";
const REFRESH = "refresh";
const PROFILE_ID = ROOT + "profileID." + VALUE;

const PREF_PATH = "extensions/prefswitch@mozdev.org";
const SWITCH_PATH = PREF_PATH + "/switch.xml";
const PROFILE_PATH = PREF_PATH + "/profile.xml";

const CHARSET = "UTF-8";
const CHARACTERS = {
	"&#38;": /&/g,
	"&#34;": /\"/g,
	"&#60;": /</g,
	"&#62;": />/g
};

function Control(overlay)
{
	this.overlay = overlay;
	this.prefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
	this.prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);

	this.getValue = function(preference, defaultValue, type)
	{
		try
		{
			switch(type)
			{
				case BOOLEAN:
					return this.prefBranch.getBoolPref(preference);

				case INTEGER:
					return this.prefBranch.getIntPref(preference);

				case STRING:
				case LIST:
				case LISTITEMS:
					return this.prefBranch.getCharPref(preference);
			}
		}
		catch(e)
		{
			if (!defaultValue) return null;
			this.setValue(preference, defaultValue, type);
		}
		return this.getValue(preference, null, type);
	};

	this.setValue = function(preference, value, type)
	{
		try
		{
			switch(type)
			{
				case BOOLEAN:
					this.prefBranch.setBoolPref(preference, "" + value == "true");
					break;

				case INTEGER:
					this.prefBranch.setIntPref(preference, value);
					break;

				case STRING:
					this.prefBranch.setCharPref(preference, value);
					break;

				case RESET:
					this.prefBranch.clearUserPref(preference);
					break;

				case LIST:
					this.prefBranch.setCharPref(preference, value);
					break;

				case LISTITEMS:
					var p = this.getValue(preference, value, STRING);
					var a = value.split(" ");
					var t = "";
					for (var i = 0; i < a.length; i++)
					{
						if (a[i] != "" && !p.match(new RegExp("(^|\\s)" + a[i].replace(/(\W)/g, "\\$1") + "(\\s|$)")))
						{
							if (t != "" || p != "") t += " ";
							t += a[i];
						}
					}
					if (t != "") this.prefBranch.setCharPref(preference, p + t);
					break;
			}
		}
		catch(e){}
	};

	this.update = function(type, element)
	{
		if (this.overlay)
		{
			switch(type)
			{
				case UPDATE_ALL: this.overlay.updateAll(); break;
				case UPDATE_ONE: this.overlay.updateOne(element); break;
				case UPDATE_PROFILES: this.overlay.updateProfiles();
			}
		}
		else
		{
			var value = this.getValue(ROOT + REFRESH + "." + VALUE, true, BOOLEAN);
			this.setValue(ROOT + REFRESH + "." + VALUE, !value, BOOLEAN);
		}
	};

	this.encode = function(string)
	{
		for (var ch in CHARACTERS) string = string.replace(CHARACTERS[ch], ch);
		return string;
	};

	this.getSwitches = function()
	{
		var doc = this.getDoc(SWITCH_PATH);
		if (!doc) return [];
		if (doc.documentElement.nodeName == "parsererror")
		{
			dump("Prefswitch::Control.getSwitches(): Error while parsing.\n");
			return [];
		}
		var aResult = [];
		var iSwitch = doc.evaluate("./prefswitch/switch", doc, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
		var nSwitch = iSwitch.iterateNext();
		while(nSwitch)
		{
			var mySwitch = new Switch(
				nSwitch.getAttribute("id"),
				nSwitch.getAttribute("name"),
				nSwitch.getAttribute("initial"));
			var iState = doc.evaluate("./state", nSwitch, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
			var nState = iState.iterateNext();
			while(nState)
			{
				var state = new State(
					nState.getAttribute("id"),
					nState.getAttribute("name"),
					nState.getAttribute("description"),
					nState.getAttribute("tooltip"),
					nState.getAttribute("color"));
				var iPreference = doc.evaluate("./preference", nState, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
				var nPreference = iPreference.iterateNext();
				while(nPreference)
				{
					state.addPreference(new Preference(
						nPreference.getAttribute("name"),
						nPreference.getAttribute("type"),
						nPreference.getAttribute("value"),
						nPreference.getAttribute("description")));
					nPreference = iPreference.iterateNext();
				}
				mySwitch.addState(state);
				nState = iState.iterateNext();
			}
			aResult.push(mySwitch);
			nSwitch = iSwitch.iterateNext();
		}
		return aResult;
	};

	this.saveSwitches = function(data) {this.saveToFile(data, SWITCH_PATH);};

	this.getProfiles = function()
	{
		var doc = this.getDoc(PROFILE_PATH);
		if (!doc) return [];
		if (doc.documentElement.nodeName == "parsererror")
		{
			dump("Prefswitch::Control.getProfiles(): Error while parsing.\n");
			return [];
		}
		var aResult = [];
		var iProfile = doc.evaluate("./prefswitch/profile", doc, null, XPathResult.UNORDERED_NODE_ITERATOR_TYPE, null);
		var nProfile = iProfile.iterateNext();
		while(nProfile)
		{
			var profile = new Profile(
				nProfile.getAttribute("id"),
				nProfile.getAttribute("name"));
			var iFeature = doc.evaluate("./feature", nProfile, null, XPathResult.UNORDERED_NODE_ITERATOR_TYPE, null);
			var nFeature = iFeature.iterateNext();
			while(nFeature)
			{
				profile.setFeature(nFeature.getAttribute("key"), nFeature.getAttribute("value"), nFeature.getAttribute("show"), nFeature.getAttribute("disabled"));
				nFeature = iFeature.iterateNext();
			}
			aResult.push(profile);
			nProfile = iProfile.iterateNext();
		}
		return aResult;	
	};

	this.saveProfiles = function(data) {this.saveToFile(data, PROFILE_PATH);};

	this.getDoc = function(path)
	{
		var file = this.getFile(path);
		if (!file) return null;
		var data = this.getData(file);
		var parser = new DOMParser();
		return parser.parseFromString(data, "text/xml");
	};

	this.getFile = function(path)
	{
		var file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
		var aPath = path.split(/\/|\\/);
		for (var i = 0; i < aPath.length; i++) file.append(aPath[i]);
		if (file.isFile()) return file;
		else return null;
	};

	this.getDirectory = function(path)
	{
try{
		var directory = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
		var aPath = path.split(/\/|\\/);
		for (var i = 0; i < aPath.length; i++) directory.append(aPath[i]);
		if (directory.isDirectory()) return directory;
}catch(e){}
		return null;
	};

	this.getData = function(file)
	{
		var fiStream = Components.classes['@mozilla.org/network/file-input-stream;1'].createInstance(Components.interfaces.nsIFileInputStream);
		fiStream.init(file, 0x01, 00004, null);
		var siStream = Components.classes['@mozilla.org/scriptableinputstream;1'].createInstance(Components.interfaces.nsIScriptableInputStream);
		siStream.init(fiStream);
		var data = siStream.read(siStream.available());
		var suc = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
		suc.charset = CHARSET;
		var data = suc.ConvertToUnicode(data);
		siStream.close();
		fiStream.close();
		return data;
	};

	this.saveToFile = function(data, path)
	{
		var aPath = path.split(/\/|\\/);
		var file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
		for (var i = 0; i < aPath.length; i++) file.append(aPath[i]);
		var fos = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		fos.init(file, 0x02 | 0x08 | 0x20, 0664, 0);
		var parser = new DOMParser();
		var doc = parser.parseFromString(data, "text/xml");
		var serializer = new XMLSerializer();
		serializer.serializeToStream(doc, fos, CHARSET);
		fos.close();
	};
}
